VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsDDClient"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
'A Class to encapsulate an instance of a DDClient control which can have one
'conversation and one outstanding request.
'Asynchronous requests are assumed to
'be for text which goes into a text box
'Synchronous requests return the string
'
'To use the control in this way, set a reference to the control.
'You may need to browse for the OCX file.

Option Explicit

Private WithEvents DDEClient As DDCDem35.DDECL
Attribute DDEClient.VB_VarHelpID = -1
Dim ReqTransKey As String
Dim lConvKey As String
Dim mTextControl As TextBox

Private Sub Class_Initialize()
   Set DDEClient = CreateObject("DDCDem35.DDECL")
   DDEClient.Initialise

  'At the moment there is no transaction outstanding or conversation
  ReqTransKey = ""
  lConvKey = ""
  
End Sub
'
'Attempt to connect to the server specified
'
Public Function Connect(Service As String, Topic As String) As Boolean

'Fail if there is already a conversation
If lConvKey <> "" Then
  Connect = False
  Exit Function
End If

lConvKey = DDEClient.Connect(Service, Topic)
If lConvKey = DDEClient.FailedReturnString Then
  lConvKey = ""
  Connect = False
Else
  Connect = True
End If

End Function
'
'Disconnect the conversation
'
Public Function Disconnect() As Boolean

'Fail if there is no conversation
If lConvKey = "" Then
  Disconnect = False
  Exit Function
End If

Call DDEClient.Disconnect(lConvKey)
Disconnect = True

End Function
'
'Request the data item specified asynchronously to be put into a text box
'
Public Function RequestAsync(Item As String, Destination As TextBox) As Boolean

'Fail if there is no conversation
If lConvKey = "" Then
  RequestAsync = False
  Exit Function
End If

'If there is an outstanding transaction cancel it
If ReqTransKey <> "" Then
  DDEClient.Conversations(lConvKey).AbandonTransaction (ReqTransKey)
  ReqTransKey = ""
End If

ReqTransKey = DDEClient.Conversations(lConvKey).RequestA(Item)
If ReqTransKey = DDEClient.FailedReturnString Then
  RequestAsync = False
  Set mTextControl = Nothing
Else
  Set mTextControl = Destination
  RequestAsync = True
End If

End Function
'
'Request data to be returned immediately
'
Public Function Request(Item As String) As String

Dim DataObj As DDEData
Dim Result As String
Dim Pos As Long

'Fail if there is no conversation
If lConvKey = "" Then
  Request = False
  Exit Function
End If

'If there is an outstanding transaction cancel it
If ReqTransKey <> "" Then
  DDEClient.Conversations(lConvKey).AbandonTransaction (ReqTransKey)
  ReqTransKey = ""
  Set mTextControl = Nothing
End If

Set DataObj = DDEClient.Conversations(lConvKey).Request(Item, 5000)

If DataObj Is Nothing Then
  Request = ""
Else
  Result = DataObj.DataString
  
  'Allow for the fact that there may be rubbish after the string
  Pos = InStr(Result, Chr$(0))
  If Pos > 0 Then Result = Left$(Result, Pos - 1)
  Request = Result
End If

End Function

'
'Handle the case that the server disconnected
'
Private Sub DDEClient_DDEDisconnect(ByVal ConvKey As String, Conv As DDCDem35.Conversation)

If lConvKey = ConvKey Then
  ConvKey = ""
  ReqTransKey = ""
  Set mTextControl = Nothing
  End If

End Sub
Private Sub DDEClient_TransactionRequestCompleted(ByVal TransKey As String, _
    Trans As Transaction, ByVal ConvKey As String, Conv As Conversation, ByVal Item As String, Data As DDEData)

Dim Result As String
Dim Pos As Long

If TransKey = ReqTransKey Then
  ReqTransKey = ""
  Result = Data.DataString
  
  'Allow for the fact that there may be rubbish after the string
  Pos = InStr(Result, Chr$(0))
  If Pos > 0 Then Result = Left$(Result, Pos - 1)
  mTextControl.Text = Result
End If
  
Set mTextControl = Nothing

End Sub
Public Sub DoUnload()

If Not DDEClient Is Nothing Then Call DDEClient.Uninitialise
Set DDEClient = Nothing
Set mTextControl = Nothing

End Sub
